import { apiConfig, defaultConfig } from "../lib/player/config";
import { ModalFormData } from "@minecraft/server-ui";
export const configPanel = new class configPanel {
    open(player) {
        const config = apiConfig.get(player);
        new ModalFormData()
            .title("ui.veinminer:config.title")
            .toggle("ui.veinminer:config.show_selection", config.showSelection)
            .dropdown("ui.veinminer:config.function_type", functionList.map(value => `ui.veinminer:config.function_type.${value}`), functionList.findIndex(value => value == config.functionType))
            .textField("ui.veinminer:config.max_size", "ui.veinminer:config.max_size_hover", String(config.maxSize))
            .toggle("ui.veinminer:config.quick_settings", config.quick)
            .show(player).then(r => {
            if (r.canceled)
                return player.onScreenDisplay.setActionBar({ translate: "warning.veinminer:config_canceled" });
            if (r.formValues == undefined)
                return;
            const newConfig = this.processInput(r.formValues);
            apiConfig.set(player, newConfig);
            if (config.quick == false && newConfig.quick == true)
                player.sendMessage({ translate: "warning.veinminer:quick_setting_enabled" });
            player.onScreenDisplay.setActionBar({ translate: "warning.veinminer:config_complete" });
        });
    }
    processInput(inputs) {
        const newInput = { ...defaultConfig };
        const [particle, type, size, quick] = inputs;
        if (typeof particle == "boolean")
            newInput.showSelection = particle;
        newInput.functionType = validateFunctionType(type);
        newInput.maxSize = validateMaxSize(size);
        if (typeof quick == "boolean")
            newInput.quick = quick;
        return newInput;
    }
};
function validateFunctionType(input) {
    if (typeof input !== "number")
        return defaultConfig.functionType;
    function isFunctionType(value) { return functionList.includes(value); }
    if (typeof functionList[input] !== "string" || !isFunctionType(functionList[input]))
        return defaultConfig.functionType;
    return functionList[input];
}
function validateMaxSize(input) {
    if (typeof input !== "string")
        return defaultConfig.maxSize;
    const num = parseInt(input);
    if (isNaN(num))
        return defaultConfig.maxSize;
    if (num < 1)
        return 1;
    if (num > 64)
        return 64;
    return num;
}
const functionList = ["off", "all", "3x3_tunnel", "mining_tunnel", "escape_tunnel"];
