import { BlockVolume } from "@minecraft/server";
import { apiConfig } from "../player/config";
import { apiVec3 } from "../math/vector";
export const apiValidBlocks = new class apiValidBlocks {
    getBlocks(player, id, pos) {
        const config = apiConfig.get(player);
        switch (config.functionType) {
            case "mining_tunnel": {
                return this.getSmallTunnel(player, id, pos, false);
            }
            case "escape_tunnel": {
                return this.getSmallTunnel(player, id, pos);
            }
            case "3x3_tunnel": {
                return this.getLargeTunnel(player, id, pos);
            }
            default: {
                return this.getAll(player, id, pos);
            }
        }
    }
    getAll(player, id, pos) {
        const config = apiConfig.get(player);
        const wood = id.some(i => i.includes("_log"));
        const size = wood ? 5 : 3;
        const blocks = player.dimension.getBlocks(new BlockVolume(apiVec3.offset(pos, apiVec3.create(-size, -size, -size)), apiVec3.offset(pos, apiVec3.create(size, wood ? 30 : size, size))), { includeTypes: id });
        const valid = [...blocks.getBlockLocationIterator()]
            .map(value => ({ pos: value, queue: apiVec3.distance(pos, value) }))
            .sort((a, b) => a.queue - b.queue);
        const specify = this.getAround(valid.slice(0, 1), valid.slice(1));
        return specify
            .filter((_, index) => index < config.maxSize * (wood ? 2 : 1))
            .sort((a, b) => a.queue - b.queue);
    }
    getLargeTunnel(player, id, pos) {
        const degress = player.getRotation().y + 180;
        const direction = apiVec3.multiply(apiVec3.offsetDirection[directions[degress < 45 || degress > 315 ? 0 : degress > 45 && degress < 135 ? 1 : degress > 135 && degress < 225 ? 2 : 3]], 16);
        const offset = offsets[degress < 45 || degress > 315 ? 0 : degress > 45 && degress < 135 ? 1 : degress > 135 && degress < 225 ? 0 : 1];
        const blocks = player.dimension.getBlocks(new BlockVolume(apiVec3.offset(pos, offset[0]), apiVec3.offset(apiVec3.offset(pos, offset[1]), direction)), { includeTypes: id });
        const valid = [...blocks.getBlockLocationIterator()]
            .map(value => ({ pos: value, queue: apiVec3.distance(pos, value) }))
            .sort((a, b) => a.queue - b.queue);
        const specify = this.getAround(valid.slice(0, 1), valid.slice(1));
        return specify;
    }
    getSmallTunnel(player, id, pos, up = true) {
        const updown = up ? "Up" : "Down";
        const degress = player.getRotation().y + 180;
        const directionH = apiVec3.offsetDirection[directions[degress < 45 || degress > 315 ? 0 : degress > 45 && degress < 135 ? 1 : degress > 135 && degress < 225 ? 2 : 3]];
        const directionV = apiVec3.offsetDirection[updown];
        const blocks = [];
        for (let i = 0; i < 15; i++) {
            const offset = apiVec3.offset(pos, apiVec3.offset(apiVec3.multiply(directionH, i), apiVec3.multiply(directionV, i)));
            try {
                const block = player.dimension.getBlock(offset);
                if (!block)
                    return blocks;
                if (!id.includes(block.typeId))
                    return blocks;
                blocks.push({ pos: block.location, queue: i });
            }
            catch (e) {
                console.log(e);
                return blocks;
            }
        }
        return blocks;
    }
    getAround(aproved, testing) {
        const removed = [];
        aproved = [...new Set(aproved)];
        aproved.map(value => {
            aproved.push(...testing.filter((value2, index) => {
                if (apiVec3.distance(value.pos, value2.pos) < 1.75) {
                    removed.push(index);
                    return true;
                }
                return false;
            }));
        });
        const remove = [...new Set(removed)];
        const nextTest = testing.filter((_, index) => !remove.includes(index));
        if (remove.length > 0)
            return this.getAround(aproved, nextTest);
        return aproved;
    }
};
const directions = ["North", "East", "South", "West"];
const offsets = [
    [{ x: 1, y: 1, z: 0 }, { x: -1, y: -1, z: 0 }],
    [{ x: 0, y: 1, z: 1 }, { x: 0, y: -1, z: -1 }]
];
