import { world, system, ItemStack, Dimension, Vector3 } from "@minecraft/server"
import { apiItemDurability } from "../lib/item/durability"
import { apiValidBlocks } from "../lib/block/validBlocks"
import { apiBlock } from "../lib/block/apiBlock"
import { apiConfig } from "../lib/player/config"
import { apiVec3 } from "../lib/math/vector"
import { removeBorder } from "./hitBlock"
import { apiNumbers } from "../lib/math/numbers"

world.beforeEvents.playerBreakBlock.subscribe(({player, block, itemStack: item}) => {
  if(!player.isSneaking || apiConfig.get(player).functionType == "off") return
  const ids = apiBlock.transformId(block.typeId)
  const enchant = item?.getComponent("enchantable")

  const blocks = apiValidBlocks.getBlocks(player, ids, block.location).slice(1)

  system.runJob(removeBorder(player.dimension.getEntities({tags: [`vein_miner_owner:${player.nameTag}`]})))
  system.runJob(destroy(block.dimension, blocks, {id: ids, type: { id: player.getGameMode() == "creative" ? "creative" : enchant?.hasEnchantment("minecraft:silk_touch") ? "silk" : enchant?.hasEnchantment("minecraft:fortune") ? "fortune" : undefined, item: item }}))
  if(item) system.run(() => { apiItemDurability.remove(player, item, blocks.length) })
})

function* destroy(dimension: Dimension, list: {pos: Vector3, queue: number}[], execute: execution): Generator<void> {
  const hasSilk = execute.id.find(value => silkList.includes(value))
  const silkItem = hasSilk ? apiBlock.transformId(hasSilk)[0] : hasSilk

  const hasFortune = execute.id.find(value => Object.keys(fortuneList).includes(value))
  const forturneItem = hasFortune ? apiBlock.transformId(hasFortune)[0] : hasFortune

  if(execute.type?.id == "creative") return list.map(value => {
    dimension.setBlockType(value.pos, "minecraft:air")
  })
  if(execute.type?.id == "silk" || (execute.type?.item?.typeId == "minecraft:shears" && execute.id.some(value => value.includes("_leaves") || value.includes(":vine")))){
    if(hasSilk) return list.map(value => {
      dimension.setBlockType(value.pos, "minecraft:air")
      dimension.spawnItem(new ItemStack(silkItem ? silkItem : hasSilk), apiVec3.centerBlock(value.pos))
    })
  }
  if(execute.type?.id == "fortune"){
    const item = forturneItem ? fortuneList[forturneItem] : undefined
    const enchant = execute.type.item?.getComponent("enchantable")?.getEnchantment("minecraft:fortune")
    const level = !enchant ? 0 : enchant.level
    if(hasFortune) list.map(value => {
      if(Math.random() >= 2/(level)) dimension.spawnItem(new ItemStack(forturneItem ? item ? item.id : forturneItem : hasFortune, Math.floor(apiNumbers.random({min: item?.min ? item.min : 1, max: item?.max ? item.max : 3}))), apiVec3.centerBlock(value.pos))
    })
  }
  for(const select of list){
    dimension.runCommand(`setblock ${select.pos.x} ${select.pos.y} ${select.pos.z} air destroy`)
    yield
  }
}

const silkList = [
  "minecraft:acacia_leaves",
  "minecraft:azalea_leaves",
  "minecraft:azalea_leaves_flowered",
  "minecraft:birch_leaves",
  "minecraft:black_stained_glass",
  "minecraft:black_stained_glass_pane",
  "minecraft:blue_ice",
  "minecraft:blue_stained_glass",
  "minecraft:blue_stained_glass_pane",
  "minecraft:bookshelf",
  "minecraft:brown_stained_glass",
  "minecraft:brown_stained_glass_pane",
  "minecraft:cherry_leaves",
  "minecraft:clay",
  "minecraft:coal_ore",
  "minecraft:copper_ore",
  "minecraft:cyan_stained_glass",
  "minecraft:cyan_stained_glass_pane",
  "minecraft:dark_oak_leaves",
  "minecraft:deepslate",
  "minecraft:deepslate_coal_ore",
  "minecraft:deepslate_copper_ore",
  "minecraft:deepslate_diamond_ore",
  "minecraft:deepslate_emerald_ore",
  "minecraft:deepslate_gold_ore",
  "minecraft:deepslate_iron_ore",
  "minecraft:deepslate_lapis_ore",
  "minecraft:diamond_ore",
  "minecraft:emerald_ore",
  "minecraft:glass",
  "minecraft:glass_pane",
  "minecraft:gold_ore",
  "minecraft:grass",
  "minecraft:grass_path",
  "minecraft:gray_stained_glass",
  "minecraft:gray_stained_glass_pane",
  "minecraft:green_stained_glass",
  "minecraft:green_stained_glass_pane",
  "minecraft:ice",
  "minecraft:iron_ore",
  "minecraft:jungle_leaves",
  "minecraft:lapis_ore",
  "minecraft:light_blue_stained_glass",
  "minecraft:light_blue_stained_glass_pane",
  "minecraft:light_gray_stained_glass",
  "minecraft:light_gray_stained_glass_pane",
  "minecraft:lime_stained_glass",
  "minecraft:lime_stained_glass_pane",
  "minecraft:lit_deepslate_redstone_ore",
  "minecraft:lit_redstone_ore",
  "minecraft:magenta_stained_glass",
  "minecraft:magenta_stained_glass_pane",
  "minecraft:mangrove_leaves",
  "minecraft:mycelium",
  "minecraft:nether_gold_ore",
  "minecraft:oak_leaves",
  "minecraft:orange_stained_glass",
  "minecraft:orange_stained_glass_pane",
  "minecraft:packed_ice",
  "minecraft:pale_oak_leaves",
  "minecraft:pink_stained_glass",
  "minecraft:pink_stained_glass_pane",
  "minecraft:podzol",
  "minecraft:purple_stained_glass",
  "minecraft:purple_stained_glass_pane",
  "minecraft:quartz_ore",
  "minecraft:red_stained_glass",
  "minecraft:red_stained_glass_pane",
  "minecraft:spruce_leaves",
  "minecraft:stone",
  "minecraft:vine",
  "minecraft:white_stained_glass",
  "minecraft:white_stained_glass_pane",
  "minecraft:yellow_stained_glass",
  "minecraft:yellow_stained_glass_pane"
]

const fortuneList: { [key: string]: {id: string, min?: number, max?: number}} = {
  "minecraft:coal_ore": { id: "minecraft:coal" },
  "minecraft:copper_ore": { id: "minecraft:raw_copper", min: 2, max: 5 },
  "minecraft:deepslate_coal_ore": { id: "minecraft:coal" },
  "minecraft:deepslate_copper_ore": { id: "minecraft:raw_copper", min: 2, max: 5 },
  "minecraft:deepslate_diamond_ore": { id: "minecraft:diamond" },
  "minecraft:deepslate_emerald_ore": { id: "minecraft:emerald" },
  "minecraft:deepslate_gold_ore": { id: "minecraft:raw_gold" },
  "minecraft:deepslate_iron_ore": { id: "minecraft:raw_iron" },
  "minecraft:deepslate_lapis_ore": { id: "minecraft:lapis_lazuli", min: 4, max: 9 },
  "minecraft:deepslate_redstone_ore": { id: "minecraft:redstone", min: 4, max: 5 },
  "minecraft:diamond_ore": { id: "minecraft:diamond" },
  "minecraft:emerald_ore": { id: "minecraft:emerald" },
  "minecraft:gold_ore": { id: "minecraft:raw_gold" },
  "minecraft:iron_ore": { id: "minecraft:raw_iron" },
  "minecraft:lapis_ore": { id: "minecraft:lapis_lazuli", min: 4, max: 9 },
  "minecraft:lit_deepslate_redstone_ore": { id: "minecraft:redstone", min: 4, max: 5 },
  "minecraft:lit_redstone_ore": { id: "minecraft:redstone", min: 4, max: 5 },
  "minecraft:nether_gold_ore": { id: "minecraft:nugget", min: 2, max: 6 },
  "minecraft:quartz_ore": { id: "minecraft:quartz" },
  "minecraft:redstone_ore": { id: "minecraft:redstone", min: 4, max: 5 }
}

interface execution {
  id: string[],
  type?: {
    id: "creative" | "silk" | "fortune" | undefined,
    item: ItemStack | undefined
  }
}