import { world, system, Dimension, Entity, Player, Vector3 } from "@minecraft/server"
import { apiValidBlocks } from "../lib/block/validBlocks"
import { apiBlock } from "../lib/block/apiBlock"
import { apiConfig } from "../lib/player/config"
import { apiTimer } from "../lib/player/timer"
import { apiVec3 } from "../lib/math/vector"

world.afterEvents.entityHitBlock.subscribe(({damagingEntity: entity, hitBlock: block}) => {
  const player = entity as Player
  const config = apiConfig.get(player)

  if(!config.showSelection || config.functionType == "off") return
  if(!player.isSneaking) return

  let testCooldown = true
  const lastPos = player.getDynamicProperty("veinminer:last_selection")
  if(!apiVec3.isValid(lastPos)){
    testCooldown = false
  } else if(!apiVec3.compare(lastPos, block.location)){
    system.runJob(removeBorder(player.dimension.getEntities({tags: [`vein_miner_owner:${player.nameTag}`]})))
    testCooldown = false
  }
  player.setDynamicProperty("veinminer:last_selection", block.location)

  if(testCooldown){
    if(!apiTimer.checkCooldown(player, "veinminer:timer_select_block").finished) return
    apiTimer.setCooldown(player, "veinminer:timer_select_block", 2)
  }

  if(block.typeId == "minecraft:air") return

  const valid = apiValidBlocks.getBlocks(player, apiBlock.transformId(block.typeId), block.location)
  system.runJob(spawnBorder(player, block.dimension, valid))
})

function* spawnBorder(player: Player, dimension: Dimension, list: {pos: Vector3, queue: number}[]): Generator<void> {
  for(const selected of list){
    const entity = dimension.spawnEntity("veinminer:selected_block", selected.pos)
    entity.addTag(`vein_miner_owner:${player.nameTag}`)
    const repeater = system.runInterval(() => {
      try { entity.playAnimation("animation.selected_block.offset", {players: [`${player.nameTag}`]}) } catch {}
    }, 1)
    system.runTimeout(() => { system.clearRun(repeater) }, 2)
    yield
  }
}

export function* removeBorder(list: Entity[]): Generator<void> {
  for(const selected of list){
    try { selected.remove() } catch {}
    yield
  }
}