import { world, Vector3 } from "@minecraft/server"

export const apiVec3 = new class apiVec3 {
  public offsetDirection = {
    "East": {x: 1, y: 0, z: 0},
    "West": {x: -1, y: 0, z: 0},
    "Down": {x: 0, y: -1, z: 0},
    "Up": {x: 0, y: 1, z: 0},
    "North": {x: 0, y: 0, z: -1},
    "South": {x: 0, y: 0, z: 1}
  }

  create(x = 0, y = 0, z = 0): Vector3 { return { x: x, y: y, z: z} }

  divide(vector: Vector3, divider: number): Vector3 {
    return {
      x: vector["x"] / divider,
      y: vector["y"] / divider,
      z: vector["z"] / divider
    }
  }

  multiply(vector: Vector3, amount: number): Vector3 {
    return {
      x: vector["x"] * amount,
      y: vector["y"] * amount,
      z: vector["z"] * amount
    }
  }

  offset(vector: Vector3, offset: Vector3): Vector3 {
    return {
      x: vector["x"] + offset["x"],
      y: vector["y"] + offset["y"],
      z: vector["z"] + offset["z"]
    }
  }

  floor(vector: Vector3): Vector3 {
    return {
      x: Math.floor(vector["x"]),
      y: Math.floor(vector["y"]),
      z: Math.floor(vector["z"])
    }
  }

  reduce(vector: Vector3, value: number, on: "x" | "y" | "z"): Vector3 {
    const newVec = { x: vector["x"], y: vector["y"], z: vector["z"] }
    newVec[on] += value
    return newVec
  }

  compare(vec1: Vector3, vec2: Vector3): boolean {
    if(vec1.x != vec2.x || vec1.y != vec2.y || vec1.z != vec2.z) return false
    return true
  }

  distance(vec1: Vector3, vec2: Vector3, negative = false): number {
    if(negative) return (vec1.x - vec2.x) + (vec1.y - vec2.y) + (vec1.z - vec2.z)
    return Math.sqrt((vec1.x - vec2.x) ** 2 + (vec1.y - vec2.y) ** 2 + (vec1.z - vec2.z) ** 2)
  }

  distanceX(vec1: Vector3, vec2: Vector3, negative = false): number {
    if(negative) return vec1.x - vec2.x
    return Math.sqrt((vec1.x - vec2.x) ** 2)
  }

  distanceY(vec1: Vector3, vec2: Vector3, negative = false): number {
    if(negative) return vec1.y - vec2.y
    return Math.sqrt((vec1.y - vec2.y) ** 2)
  }

  distanceZ(vec1: Vector3, vec2: Vector3, negative = false): number {
    if(negative) return vec1.z - vec2.z
    return Math.sqrt((vec1.z - vec2.z) ** 2)
  }

  isValid(obj: any): obj is { x: number, y: number, z: number } {
    if(obj == undefined) return false
    const keys = Object.keys(obj)
    return (keys.length === 3 && order.every(k => keys.includes(k))) && (order.every(k => typeof obj[k] === 'number'))
  }

  organize(vector: Vector3): Vector3 {
    const { x, y, z } = vector
    return { x: x, y: y, z: z }
  }

  decompose(vector: Vector3): number[] { return Object.values(vector) }

  centerBlock(vector: Vector3): Vector3 { return this.offset(this.floor(vector), this.create(0.5, 0.5, 0.5)) }
}

const order = ['x', 'y', 'z'] as const